/*
 * filename: src/addimage.c
 * author: Pablo Yanez Trujillo <yanezp@imtek.de>
 */

#include <stdio.h>
#include <sqlite3.h>
#include <sstrings.h> /* includes string.h, stdlib.h, ctype.h */

#include "addimage.h"

static int getid(void *NotUsed, int argc, char **argv, char **azColName)
{
	int *id = NotUsed;
	
	*id = atoi(argv[0]);

	return 0;
}

int db_addimage(const char* path, SLL* args)
{
	SLL* list;
	char *cat_name, *image_name, *image_path, *quick_desc, *long_desc, *tmp;
	const char *type, *author, *year, *language;
	char sql_command [1024];
	char *dbErrMsg = NULL;
        sqlite3* db;
	int got, id = -1;
	FILE* f;
	const char* questions[] = { "Image Name: ", "Category name: ", "Type (CD/DVD) [CD]: ", "Year: ", 
				    "Author: ", "Language: ", "Absolute path of ISO file: ",
				    "Quick description: ", "Long Description: ", "", "", "", "", "", "", "r", "", " " };
	if(!path || !args) return 0;

	/* order: category, name, type, year, author, isopath, desc, longdesc */
	list = userinput(questions, 9, args);

	if(!list)
		return 0;
	
	/* check input */

	got = 0;

	image_name = getSLL(list, 0, &got);

        if(!got)
        {
                fprintf(stderr, "Unexpected error\n");
                cleanupSLL(list);
                return 0;
        }

        if(!strcmp(image_name, ""))
        {
                fprintf(stderr, "Image name must not be blank.\n");
                cleanupSLL(list);
                return 0;
        }

        if(strlen(image_name) > 10)
        {
                fprintf(stderr, "Image name is too long. Only the first 10 characters are saved.\n");
                image_name[10] = 0;
        }

	tmp = prepare_input(image_name);

	if(tmp)
		image_name = tmp;

	
	cat_name = getSLL(list, 1, &got);

	if(!got)
	{
		fprintf(stderr, "Unexpected error\n");
		cleanupSLL(list);
		return 0;
	}

	if(!strcmp(cat_name, ""))
	{
		fprintf(stderr, "Category name must bot be blank.\n");
		cleanupSLL(list);
		return 0;
	}

	tmp = prepare_input(cat_name);

	if(tmp)
		cat_name = tmp;

	type = getSLL(list, 2, &got);

	if(!got)
        {
                fprintf(stderr, "Unexpected error\n");
                cleanupSLL(list);
                return 0;
        }

	if(!strcmp(type, ""))
		type = "CD";


	if(strcmp(type, "CD") && strcmp(type, "DVD"))
	{
		fprintf(stderr, "'%s' is not a valid image type.\n", type);
		cleanupSLL(list);
		return 0;
	}

	year = getSLL(list, 3, &got);

	if(!got)
        {
                fprintf(stderr, "Unexpected error\n");
                cleanupSLL(list);
                return 0;
        }

	if(!strcmp(year, ""))
		year = "Unbekannt";
	else {
		tmp = prepare_input(year);

		if(tmp)
			year = tmp;
	}


	author = getSLL(list, 4, &got);

        if(!got)
        {
                fprintf(stderr, "Unexpected error\n");
                cleanupSLL(list);
                return 0;
        }

	if(!strcmp(author, ""))
		author = "Unbekannt";
	else {
		tmp = prepare_input(author);

		if(tmp)
			author = tmp;
	}

	language = getSLL(list, 5, &got);

	if(!got)
	{
		fprintf(stderr, "Unexpected error\n");
		cleanupSLL(list);
		return 0;
	}

	if(!strcmp(language, ""))
		language = "Unbekannt";
	else {
		tmp = prepare_input(language);

		if(tmp)
			language = tmp;
	}

	image_path = getSLL(list, 6, &got);

        if(!got)
        {
                fprintf(stderr, "Unexpected error\n");
                cleanupSLL(list);
                return 0;
        }

	if(!strcmp(image_path, ""))
	{
		fprintf(stderr, "The image path must not be blank\n");
		cleanupSLL(list);
		return 0;
	}

	f = fopen(image_path, "r");

	if(!f)
	{
		fprintf(stderr, "'%s' does not exist/cannot be opened\n", image_path);
		cleanupSLL(list);
		return 0;
	}

	fclose(f);

	tmp = prepare_input(image_path);

	if(tmp)
		image_path = tmp;

	quick_desc = getSLL(list, 7, &got);

	if(!got)
        {
                fprintf(stderr, "Unexpected error\n");
                cleanupSLL(list);
                return 0;
        }

	if(!strcmp(quick_desc, ""))
	{
		fprintf(stderr, "Quick description must not be blank\n");
		cleanupSLL(list);
		return 0;
	}

	tmp = prepare_input(quick_desc);

	if(tmp)
		quick_desc = tmp;

	long_desc = getSLL(list, 8, &got);

        if(!got)
        {
                fprintf(stderr, "Unexpected error\n");
                cleanupSLL(list);
                return 0;
        }

	tmp = prepare_input(long_desc);

	if(tmp)
		long_desc = tmp;

	sprintf(sql_command, "SELECT id FROM category WHERE name = '%s'", cat_name);

	db = db_connect(path);

	if(!db)
	{
		cleanupSLL(list);
		sqlite3_close(db);
		return 0;
	}

	got = sqlite3_exec(db, sql_command, getid, &id, &dbErrMsg);

	if(got != SQLITE_OK)
	{
		fprintf(stderr, "SQL error: %s\n", dbErrMsg);
		cleanupSLL(list);
		sqlite3_close(db);
		return 0;
	}

	if(id == -1)
	{
		fprintf(stderr, "Category '%s' does not exist.\n", cat_name);
		cleanupSLL(list);
		sqlite3_close(db);
		return 0;
	}

	/* order: category, name, type, year, author, isopath, desc, longdesc */

	sprintf(sql_command, "INSERT INTO image(cat, name, desc, longdesc, type, year, author, path, language) "
			"VALUES('%d', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s')", id, image_name,
			quick_desc, long_desc, type, year, author, image_path, language);

	got = sqlite3_exec(db, sql_command, NULL, NULL, &dbErrMsg);

	if(got != SQLITE_OK)
	{
		fprintf(stderr, "SQL error: %s\n", dbErrMsg);
		cleanupSLL(list);
		sqlite3_close(db);
		return 0;
	}

	cleanupSLL(list);
	sqlite3_close(db);
	return 1;
}
