/*
 * filename: src/main.c
 * author: Pablo Yanez Trujillo <yanezp@imtek.de>
 *
 */

#include <sqlite3.h>
#include <stdlib.h>
#include <config.h>
#include <unistd.h>
#include <sys/types.h>

#include "arg_parse.h"
#include "conf_parse.h"
#include "utils.h"

/* load modules */
#include "install.h"
#include "addcat.h"
#include "showcat.h"
#include "addimage.h"
#include "showimage.h"
#include "rmcat.h"
#include "rmimage.h"

int main(int argc, char* argv[])
{
	struct ARGS ar;
	struct CONF conf;

	int ugid[2] ={ -1, -1};


	if(argc == 1)
	{
		help(argv[0], 1);
		return EXIT_FAILURE;
	}

	ar.flag = 0;
	ar.argc = 0;
	ar.arguments = 0;

	if(!getargs(argc, argv, &ar))
	{
		cleanupSLL(ar.arguments);
		return EXIT_FAILURE;
	}

	if(!ar.flag)
	{
		help(argv[0], 1);
		cleanupSLL(ar.arguments);
		return EXIT_FAILURE;
	}

	conf.db = NULL;
	conf.uid = -1;
	conf.gid = -1;

	if(!getconf(&conf))
	{
		free(conf.db);
		cleanupSLL(ar.arguments);
		printf("error while reading config file\n");
		return EXIT_FAILURE;
	}

	/* check uid and gid */

	ugid[0] = getuid();
	ugid[1] = getgid();

	/* -c requires root */

	if(ar.flag == 1)
	{
		if(ugid[0] || ugid[1])
		{
			fprintf(stderr, "Only root may run aof-db -c\n");
			free(conf.db);
			cleanupSLL(ar.arguments);
			return EXIT_FAILURE;
		}
	} else {
		if( (ugid[0] != conf.uid) || (ugid[1] != conf.gid) )
		{
			fprintf(stderr, "Your user and group id do not match with the configured ids.\n");
			free(conf.db);
			cleanupSLL(ar.arguments);
			return EXIT_FAILURE;
		}
	}

	/* load modules */

	switch(ar.flag & ~(1 << 7))
	{
		case 1:
			/* option: -c --create */
			if(!db_install(conf.db, conf.uid, conf.gid))
			{
				free(conf.db);
				cleanupSLL(ar.arguments);
				return EXIT_FAILURE;
			}

			printf("The initial database was successfully installed on %s\n"
					"Only the configured user may run aof-db.\n", conf.db);
			break;
		case 1 << 1:
			/* option: -a --addcat */
			if(!db_addcat(conf.db, ar.arguments))
			{
				free(conf.db);
				cleanupSLL(ar.arguments);
				return EXIT_FAILURE;
			}
			break;
		case 1 << 2:
			/* option: -s --showcat */
			if(!db_showcat(conf.db, ar.arguments))
			{
				free(conf.db);
				cleanupSLL(ar.arguments);
				return EXIT_FAILURE;
			}
			break;
		case 1 << 3:
			/* option: -r rmcat */
			if(!db_rmcat(conf.db, ar.arguments, ar.flag & 1 << 7))
			{
				free(conf.db);
				cleanupSLL(ar.arguments);
				return EXIT_FAILURE;
			}
			break;
		case 1 << 4:
			/* option: -i --addimage */
			if(!db_addimage(conf.db, ar.arguments))
			{
				free(conf.db);
				cleanupSLL(ar.arguments);
				return EXIT_FAILURE;
			}
			break;
		case 1<< 5:
			/* option: -f --showimage */
			if(!db_showimage(conf.db, ar.arguments))
			{
				free(conf.db);
				cleanupSLL(ar.arguments);
				return EXIT_FAILURE;
			}
			break;
		case 1<< 6:
			/* option: -m --rmimage */
			if(!db_rmimage(conf.db, ar.arguments, ar.flag & 1 << 7))
                        {
                                free(conf.db);
                                cleanupSLL(ar.arguments);
                                return EXIT_FAILURE;
                        }
			break;
	}

	free(conf.db);
	cleanupSLL(ar.arguments);

	return EXIT_SUCCESS;
}
