/*
 * filename: src/showimage.h
 * author: Pablo Yanez Trujillo <yanezp@imtek.de>
 */

#include <stdio.h>
#include <sqlite3.h>
#include <sstrings.h> /* includes string.h, stdlib.h, ctyoe.h */

#include "showimage.h"

static int getid(void *NotUsed, int argc, char **argv, char **azColName)
{
	int* id = NotUsed;

	*id = atoi(argv[0]);

	return 0;
}

static int showimg(void *NotUsed, int argc, char **argv, char **azColName)
{
	/* SELECT name,desc,longdesc,type,year,author,path,cat,language,id FROM image WHERE cat = %d 
	 *        0    1    2        3    4    5      6    7   8	9
	 */

	printf("--------------\n- %s(%s) [%s]\nCategory: %s\nID: %s\nAuthor: %s\nYear: %s\nLanguage: %s\nPath: %s\nDescription: %s\n--------------\n\n", 
			argv[0],
			argv[1],
			argv[3],
			argv[7],
			argv[9],
			argv[5],
			argv[4],
			argv[8],
			argv[6],
			argv[2]);
	
	return 0;
}

int db_showimage(const char* path, SLL* args)
{
	SLL* list;
	char* cat_name, *tmp;
	int got;
	const char* questions[] = { "Show images of category [leave blank for all images]: ", "" };
	sqlite3* db;
	char sql_command[1024];
	char *dbErrMsg = NULL;
	int catid = -1;
	
	
	if(!path || !args)
		return 0;

	list = userinput(questions, 1, args);

	if(!list)
		return 0;

	cat_name = getSLL(list, 0, &got);

	if(!got)
	{
		fprintf(stderr, "Unexpected error\n");
		cleanupSLL(list);
		return 0;
	}

	tmp = prepare_input(cat_name);

	if(tmp)
		cat_name = tmp;

	db = db_connect(path);

	if(!db)
		return 0;

	if(strcmp(cat_name, ""))
	{
		sprintf(sql_command, "SELECT id FROM category WHERE name = '%s'", cat_name);

		got = sqlite3_exec(db, sql_command, getid, &catid, &dbErrMsg);

		if(got != SQLITE_OK)
		{
			fprintf(stderr, "SQL error: %s\n", dbErrMsg);
                        cleanupSLL(list);
                        sqlite3_close(db);
			return 0;
		}

		if(catid == -1)
		{
			fprintf(stderr, "Category '%s' does not exist.\n", cat_name);
			cleanupSLL(list);
			sqlite3_close(db);
			return 0;
		}

		sprintf(sql_command, "SELECT I.name,I.desc,I.longdesc,I.type,I.year,I.author,I.path,C.name,I.language,I.id FROM image I JOIN category C ON I.cat = C.id WHERE I.cat = %d ORDER BY I.name", catid);
	} else
		sprintf(sql_command, "SELECT I.name,I.desc,I.longdesc,I.type,I.year,I.author,I.path,C.name,I.language,I.id FROM image I JOIN category C ON I.cat = C.id ORDER BY I.name");

	got = sqlite3_exec(db, sql_command, showimg, NULL, &dbErrMsg);

	if(got != SQLITE_OK)
	{
		fprintf(stderr, "SQL error: %s\n", dbErrMsg);
		cleanupSLL(list);
		sqlite3_close(db);
		return 0;
	}
	
	cleanupSLL(list);
	sqlite3_close(db);
	return 1;
}
